#ifndef	_OSUtil_
#include <:Mac #includes:OSUtil.h>
#endif
#include <:Mac #includes:pascal.h>

/*	PROCs that operate on the PixMap or the PixMap's clut must have resource
	IDs of 1000, 1020, 1040,... 1580.  PROCs that operate on the BitMap must
	have resource IDs of 2000, 2020, 2040,... 2580.  The BitMap is assumed to
	have the same dimentions as the PixMap.  All PROCs all called with a pointer
	to the Share struct and a message telling them what to do:
	
	pascal int main(AppDataPtr, message)
	SharePtr AppDataPtr;
	int message;
	{
		return(err);
	}
	
	Note:  You can place a PROC in a file name "Test PROC" in the same directory
	as Vision Lab.  If you do this its resource fork will be opened and used
	when Vision Lab runs.  There bye easing the development time for PROCs
	because you can test them without actually installing them.
*/

/*	messages that are less than 100 are reserved for the menu items in the
	PROC's hierarchical menu and should be handled as such.
*/

/*	initFlags means the PROC should set up the procFlags record.  All PROCs are
	called at application startup with this message.
*/

#define		initFlags	100

/*	doIt means simply that the PROC's menu was selected and the PROC should
	do whatever it is supposed to do.
*/

#define		doIt		101

/*	imagePlane means the BitMap PROC should image a color plane from the PixMap.
	The grayMapPtr will be set in such a way that it will map the PixMap values
	to color values instead of gray values.  You must not bring up any dialogs
	when called with this message.  You can still use the bar graph display
	however.  WARNING!!!!!  You MUST Offset the selRect on the BitMap image
	so its upper left-hand corner is at 0, 0.  This is because color imaging
	will be performed on a BitMap that is equal in size to the selRect regardless
	of the size of the PixMap.
*/

#define		imagePlane	102

/*	All PROCs that set the canRun flag are called at application startup with
	this message.  This is a good time to make sure refCon is valid, and if
	you are going to store a handle here you might want to set it to zero now
	and store the handle when the PROC is called upon to actually do its thing.
	This is also the time to make sure any hierarchical menu items are checked
	or otherwise changed.
*/

#define		initData	103

typedef struct {
	char canRun : 1;					/*	set this flag if your PROC can run on
										the current machine.
									*/
	char canBitPlane : 1;			/*	set this flag if your PROC can make a
										color bit pane from the PixMap.  For
										BitMap PROCs only.
									*/
	char needsMenu : 1;				/*	set this flag if your PROC needs its
										own hierarchical menu.  The menu should
										be stored in a resource with an ID of
										100+(PROC ID - 1000)/20 for PixMap PROCs
										and 200+(PROC ID - 2000)/20 for BitMap
										PROCs.
									*/
	char filler : 5;				/*	reserved.  Do not change. */
	char pid;						/*	used internally.  Do not change! */
} procFlags, *procFlagsPtr;

typedef struct {
	PixMapPtr pixMapPtr;			/*	pixMapPtr points to the PixMap struct
										in memory.  (See Inside Mac V for the
										description of a PixMap struct).
									*/
	BitMap *bitMapPtr;				/*	bitMapPtr points to the BitMap struct
										in memory.	(See Inside Mac I for the
										description of a BitMap struct).  The
										BitMap will initially be set to all
										white by Vision Lab on the first call
										to a BitMap PROC.
									*/
	unsigned char *grayMapPtr;		/*	grayMapPtr is an array of 256 chars in
										memory that map a pixel value to it's
										gray level value.  For example: in 4
										bit deep PixMap there might be a pixel
										with a value of 13.  To find out what
										gray level this really is we look at
										the value stored at *(grayMapPtr+13).
										This will be a value from 0 to 255. 0
										is black and 255 is white.
									*/
	Rect selRect;					/*	selRect is the rectangle that has been
										selected on the screen.  If no selection
										is present then selRect is equal to the
										dimensions of the PixMap.
									*/
	int minPixSize;					/*	minPixSize is the minimum bits needed
										per pixel.  For example: an 8 bit deep
										PixMap may only really contain 5 bits
										of information.
									*/
	int BitsChanged : 1;			/*	This flag MUST be set if you change
										any bits in the BitMap.
									*/
	int PixsChanged : 1;			/*	This flag MUST be set if you change
										any pixels in the PixMap.
									*/
	int ClutChanged : 1;			/*	This flag MUST be set if you change
										the clut of the PixMap.
									*/
	int SizeChanged : 1;			/*	This flag MUST be set if you change
										the size of the PixMap and BitMap.
										It is EXTREMELY IMPORTANT to keep the
										size of the BitMap (if it exists)
										equal to the size of the PixMap.
									*/
	int FlagFiller1 : 12;			/*	Reserved for future use - do not change!
									*/
	int ProcResID;					/*	The resource ID of the PROC being called.
										If a PROC needs any private resources
										they should be numbered from XXXX to XX99.
										Where XXXX is the PROCs resource ID.
									*/
	Handle PixHand;					/*	Handle to the pixel image in memory.  The
										handle is locked when passed to outside
										PROCs and the dereferenced pointer is stored
										in the PixMap */
	Handle BitHand;					/*	Handle to the bit image in memory.  The
										handle is locked when passed to outside
										PROCs and the dereferenced pointer is stored
										in the BitMap */
	unsigned long *tSizePtr;		/*	Total size for bar graph procedure */
	unsigned long *pSizePtr;		/*	Processed size for bar graph procedure */
	long refCon;					/*	A long value that will be preserved across
										multiple PROC calls.  It will also be
										saved and restored from the Prefs file.  Your
										PROC can store anything it wants here.  The
										value stored here will not be touched by
										other PROCs or the application.  If you store
										a handle here make sure it contains only data
										you really must have for the next operation
										and that the handle is unlocked when your PROC
										is finished.  In other words, be friendly!
									*/
	procFlags theFlags;				/*	bits flags describing what the PROC can and
										cannot do.  These flags MUST be set by the
										PROC when it receives an intialize message.
									*/
	int planeColor;					/*	If the PROC is called for imaging a color
										plane for printing this field will contain
										the color for the plane.  The values found
										here will be the same as the old QuickDraw
										color values: blackColor, whiteColor, etc.
									*/
	StringHandle errString;			/*	If you want your PROC to put up an error
										message of its own store a handle to the
										string describing the error here and return
										-209 as the error code.  (The string will
										be disposed after displaying the error).
									*/
	SysEnvRec *theWorldPtr;			/*	info about current machine */
	int	SharedVersion;				/*	Version number of this struct, currently 1 */
	int filler1;
	long filler2[24];				/*	reserved for future use */
	
	/* -=[]=--=[]=--=[]=--=[]=--=[]=--=[]=--=[]=--=[]=--=[]=--=[]=--=[]=--=[]=--=[]=-*/
	
	int (*PgetAPixel)();			/*	pascal int PgetAPixel(x, y, mapPtr)
										int x, y;
										PixMapPtr mapPtr;
										{
											return(rawPixelValue);
										}
										
										Gets a pixel value from either a BitMap or a
										PixMap.  WARNING!!! No range checking is done on
										input values!
									*/
	void (*PsetAPixel)();			/*	pascal void PsetAPixel(x, y, mapPtr, value)
										int x, y;
										PixMapPtr mapPtr;
										int value;
										{
										}
										
										Sets a pixel in either a BitMap or a PixMap to
										value.  WARNING!!! No range checking is done on
										input values!
									*/
	void (*PdoGM)();				/*	pascal void PdoGM(Position, theEvent, map, data)
										Point Position;
										EventRecord *theEvent;
										unsigned char *map;
										MapDataPtr data;
										{
										}
										
										Does a gray map more on this later.
									*/
	int (*PupdateBarGraph)();		/*	pascal int PupdateBarGraph()
										{
											return(0, or 128 if Command Period Abort);
										}
										
										Call this to draw the bar graph periodically.
										It usually returns zero, but if the command
										and period keys are held down it will return
										and one.
									*/
	void (*PsetupBarGraph)();		/*	pascal void PsetupBarGraph(theText)
										StringPtr theText;
										{
										}
										
										Call this to initialize the bar graph.  theText
										will be placed inside the bar graph and should
										say what exactly is taking so long to do.
									*/
	void (*PkillBarGraph)();		/*	pascal void PkillBarGraph()
										{
										}
										
										You *MUST* call this when when you are finished
										with the bar graph!
									*/
	int (*PgetMaxPixSize)();		/*	pascal int PgetMaxPixSize()
										{
											return(maxPixelSize);
										}
										
										returns the number of bits needed to represent
										the data in the PixMap.  This is the same number 'x'
										in the info window where x+1 -> y.
									*/
} Share, *SharePtr;

typedef struct {
	int lastX;
	int lastY;
	unsigned char brightness;
	unsigned char contrast;
} MapData, *MapDataPtr;

#define	getAPixelM(a, b, c)		CallPascalW((a), (b), (c), (*AppDataPtr->PgetAPixel))
#define	setAPixelM(a, b, c, d)	CallPascal((a), (b), (c), (d), (*AppDataPtr->PsetAPixel))
#define	doGMM(a, b, c, d)		CallPascal((a), (b), (c), (d), (*AppDataPtr->PdoGM))
#define	updateBarGraphM()		CallPascalW(*AppDataPtr->PupdateBarGraph)
#define	setupBarGraphM(a)		CallPascal((a), *AppDataPtr->PsetupBarGraph)
#define	killBarGraphM()			CallPascal(*AppDataPtr->PkillBarGraph)
#define	getMaxPixSizeM()		CallPascalW(*AppDataPtr->PgetMaxPixSize)

#define	pSizeM					(*(AppDataPtr->pSizePtr))
#define	tSizeM					(*(AppDataPtr->tSizePtr))
